@@ -14,10 +14,10 @@ import Test.Hspec
 import Control.Exception.Annotated
 import qualified Control.Exception.Safe as Safe
 import Data.Annotation
-import GHC.Stack
-import Data.Typeable
 import Data.AnnotationSpec ()
 import Data.Maybe
+import Data.Typeable
+import GHC.Stack
 
 instance Eq CallStack where
     a == b = show a == show b
@@ -37,6 +37,9 @@ pass = pure ()
 emptyAnnotation :: e -> AnnotatedException e
 emptyAnnotation = pure
 
+omitEmptyLines :: String -> [String]
+omitEmptyLines = filter (/= "") . lines
+
 spec :: Spec
 spec = do
     describe "toException" $ do
@@ -55,6 +58,51 @@ spec = do
                     SomeException $
                         AnnotatedException ["hello", "goodbye"] (SomeException TestException)
 
+    describe "displayException" $ do
+        it "is identical on SomeException" $ do
+            omitEmptyLines (displayException TestException)
+                `shouldBe` omitEmptyLines (displayException (SomeException TestException))
+        it "is reasonably nice to look at" $ do
+            omitEmptyLines (displayException (AnnotatedException [] TestException))
+                `shouldBe`
+                    [ "! AnnotatedException !"
+                    , "Underlying exception type: TestException"
+                    , "show:"
+                    , "\tTestException"
+                    , "displayException:"
+                    , "\tTestException"
+                    , "(no callstack available)"
+                    ]
+        it "is reasonably nice to look at" $ do
+            lines (displayException (AnnotatedException [Annotation @String "asdf"] TestException))
+                `shouldBe`
+                    [ "! AnnotatedException !"
+                    , "Underlying exception type: TestException"
+                    , ""
+                    , "show:"
+                    , "\tTestException"
+                    , ""
+                    , "displayException:"
+                    , "\tTestException"
+                    , "Annotations:"
+                    , "\t * Annotation @[Char] \"asdf\""
+                    , "(no callstack available)"
+                    ]
+        it "shows underlying exception type" $ do
+            Left exn <- try $ throwWithCallStack (AnnotatedException [Annotation @String "asdf"] TestException)
+            let resultLines =
+                    [ "! AnnotatedException !"
+                    , "Underlying exception type: TestException"
+                    , "show:"
+                    , "\tTestException"
+                    , "displayException:"
+                    , "\tTestException"
+                    , "Annotations:"
+                    , "\t * Annotation @[Char] \"asdf\""
+                    ]
+            take (length resultLines) (omitEmptyLines (displayException (exn :: AnnotatedException SomeException))) `shouldBe`
+                resultLines
+
     describe "AnnotatedException can fromException a" $ do
         it "different type" $ do
             fromException (toException TestException)
